package apml.contenedores;

import android.content.Context;
import android.webkit.WebView;

import androidx.test.rule.ActivityTestRule;

import org.junit.Rule;
import org.junit.Test;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

public class MainActivityTest {

    /**
     * Regla que configura la Actividad principal (MainActivity), ejecutando el hilo correcto
     */
    @Rule
    public ActivityTestRule<MainActivity> mActivityTestRule = new ActivityTestRule<MainActivity>(MainActivity.class);

    /**
     * Comprueba el aparece correctamente un mensaje Toast específico
     * Esta prueba se pasa si la orientación es vertical. Falla en caso contrario
     */
    @Test
    public void toast_muestra_texto_especifico_test() {
        onView(withText("Cambiando a orientación vertical..."))
                .inRoot(new ToastMatcher())
                .check(matches(isDisplayed()));
    }

    /**
     * Al mostrar la portada,hay que hacer click para pasar a la página de login
     */
    @Test
    public void con_un_click_muestra_pagina_login() {
        onView(withId(R.id.webview)).perform(click());
    }

    /**
     * Crea correctamente nuestra vista identificada en MainActivity como "webview"
     */
    @Test
    public void webview_ejecutado_correctamente() {
        MainActivity mActivity = mActivityTestRule.getActivity();
        WebView vista = mActivity.findViewById(R.id.webview);
        assertNotNull(vista);
    }

    /**
     * Comprueba que se usa el contexto correcto de la aplicación
     */
    @Test
    public void usa_contexto_app_test() {
        Context contexto_app = mActivityTestRule.getActivity().getBaseContext();
        assertEquals("apml.contenedores", contexto_app.getPackageName());
    }

}