package apml.contenedores;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

/**
 * Clase BaseDeDatosLocal
 */
public class BaseDeDatosLocal extends SQLiteOpenHelper {

    String db_drop_trabajador = "DROP TABLE IF EXISTS trabajador";
    String db_drop_consignatario = "DROP TABLE IF EXISTS consignatario";
    String db_drop_operacion = "DROP TABLE IF EXISTS operacion";
    String db_create_trabajador = "CREATE TABLE IF NOT EXISTS trabajador (id_trabajador TEXT PRIMARY KEY, password TEXT, nombre TEXT)";
    String db_create_consignatario = "CREATE TABLE IF NOT EXISTS consignatario (id_consignatario INTEGER PRIMARY KEY AUTOINCREMENT, nombre TEXT)";
    String db_create_operacion = "CREATE TABLE IF NOT EXISTS operacion (id_operacion INTEGER PRIMARY KEY AUTOINCREMENT, id_trabajador TEXT, id_consignatario INTEGER, matricula TEXT, vacio TEXT, origen TEXT, destino TEXT, fecha TEXT)";

    /**
     * Constructor de la base de datos
     * @param contexto El contexto
     * @param nombre   Nombre de la base de datos
     * @param factory  Factory
     * @param version  Versión de la base de datos
     */
    public BaseDeDatosLocal(Context contexto, String nombre, SQLiteDatabase.CursorFactory factory, int version) {
        super(contexto, nombre, factory, version);
    }

    /**
     * Solo se ejecuta cuando el archivo de base de datos no existe y se acaba de crear
     * @param db La base de datos donde realizar acciones
     */
    @Override
    public void onCreate(SQLiteDatabase db) {
        db.execSQL(db_drop_trabajador);
        db.execSQL(db_drop_consignatario);
        db.execSQL(db_drop_operacion);
        db.execSQL(db_create_trabajador);
        db.execSQL(db_create_consignatario);
        db.execSQL(db_create_operacion);
    }

    /**
     * Se ejecuta cuando el archivo de base de datos existe pero el número de versión almacenada es inferior al solicitado en el constructor
     * Debe actualizar el esquema de la tabla a la versión solicitada
     * @param db              La base de datos donde realizar acciones
     * @param versionAnterior Número de la versión anterior
     * @param versionNueva    Número de la nueva versión
     */
    @Override
    public void onUpgrade(SQLiteDatabase db, int versionAnterior, int versionNueva) {
        db.execSQL(db_drop_trabajador);
        db.execSQL(db_drop_consignatario);
        db.execSQL(db_drop_operacion);
        db.execSQL(db_create_trabajador);
        db.execSQL(db_create_consignatario);
        db.execSQL(db_create_operacion);
    }

    /**
     * Datos iniciales a insertar
     * @param db La base de datos donde realizar acciones
     */
    public void insercion_inicial(SQLiteDatabase db) {
        db.execSQL("DELETE FROM trabajador");
        db.execSQL("DELETE FROM consignatario");
        if (db != null) {
            db.execSQL("INSERT INTO trabajador (id_trabajador, password, nombre) VALUES ('acereno', 'acn1', 'Antonio Cereño Nieto')");
            db.execSQL("INSERT INTO trabajador (id_trabajador, password, nombre) VALUES ('ecereno', 'ecn1', 'Eduardo Cereño Nieto')");
            db.execSQL("INSERT INTO trabajador (id_trabajador, password, nombre) VALUES ('jfernandez', 'jfs1', 'Juan Antonio Fernández Sabio')");
            db.execSQL("INSERT INTO trabajador (id_trabajador, password, nombre) VALUES ('jhernandez', 'jhc1', 'Juan Antonio Hernández Coca')");
            db.execSQL("INSERT INTO consignatario (id_consignatario, nombre) VALUES (1, 'AEROMARINE')");
            db.execSQL("INSERT INTO consignatario (id_consignatario, nombre) VALUES (2, 'CARMELO MARTINEZ')");
            db.execSQL("INSERT INTO consignatario (id_consignatario, nombre) VALUES (3, 'MSC')");
            db.execSQL("INSERT INTO consignatario (id_consignatario, nombre) VALUES (4, 'SALAMA')");
        }
    }
}
