package apml.contenedores;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.webkit.JavascriptInterface;
import android.webkit.WebView;
import android.widget.Toast;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class JavaScriptInterface {

    private Context contexto;
    private SQLiteDatabase db_contenedores;
    private String id_trabajador_sesion;
    private WebView contenedores;

    /**
     * Constructor de la interfaz JavaScript
     * @param c             Contexto
     * @param db            Base de datos
     * @param id_trabajador Id del trabajador
     * @param wv            Vista web
     */
    public JavaScriptInterface(Context c, SQLiteDatabase db, String id_trabajador, WebView wv) {
        contexto = c;
        db_contenedores = db;
        id_trabajador_sesion = id_trabajador;
        contenedores = wv;
    }

    /**
     * Muestra un mensaje en formato Toast
     * @param mensaje El mensaje a mostrar
     */
    @JavascriptInterface
    public void muestra_mensaje(String mensaje) {
        Toast.makeText(contexto, mensaje, Toast.LENGTH_LONG).show();
    }

    /**
     * Muestra un mensaje en un cuadro de alerta
     * @param titulo  El titulo del cuadro
     * @param mensaje El mensaje a mostrar en el cuadro
     * @param boton   El texto del botón del cuadro
     */
    @JavascriptInterface
    public void muestra_alerta(String titulo, String mensaje, String boton) {
        AlertDialog.Builder alerta = new AlertDialog.Builder(contexto);
        alerta.setTitle(titulo);
        alerta.setMessage(mensaje);
        alerta.setPositiveButton(boton, null);
        alerta.show();
    }

    /**
     * Muestra los id_trabajador en el formulario de login
     * @return Los valores de los id_trabajador para el campo option del formulario
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public String muestra_login_id_trabajador() throws ClassNotFoundException, SQLException {
        StringBuilder option_id_trabajador = new StringBuilder();
        Cursor resultado = db_contenedores.rawQuery("SELECT id_trabajador FROM trabajador", null);
        while (resultado.moveToNext()) {
            option_id_trabajador.append("<option value=\"")
                    .append(resultado.getString(0))
                    .append("\">")
                    .append(resultado.getString(0))
                    .append("</option>").append("\n");
        }
        return option_id_trabajador.toString();
    }

    /**
     * Comprueba el login
     * @param id_trabajador El id_trabajador
     * @param password      La contraseña
     * @param orientacion   La orientacion de pantalla
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public void comprueba_login (String id_trabajador, String password, final String orientacion) throws ClassNotFoundException, SQLException {
        id_trabajador_sesion = id_trabajador;
        Cursor resultado = db_contenedores.rawQuery("SELECT id_trabajador FROM trabajador WHERE id_trabajador='" + id_trabajador + "' and password='" + password + "'", null);
        if (resultado.moveToNext()) {
            contenedores.post(new Runnable() { // Si los datos de identificación son correctos, la vista ejecuta el html correspondiente y pasa a la pantalla de operaciones
                @Override
                public void run() {
                    if(Objects.equals(orientacion, "horizontal")) {
                        contenedores.loadUrl("file:///android_asset/operaciones_horizontal.html");
                    }
                    if(Objects.equals(orientacion, "vertical")) {
                        contenedores.loadUrl("file:///android_asset/operaciones_vertical.html");
                    }
                }
            });
        } else {
            muestra_alerta("AVISO - Autenticación incorrecta", "Comprueba los datos y vuelve a intentarlo\n\nSi el error persiste, contacta con la División de Sistemas de Información y Comunicaciones", "ACEPTAR");
        }
    }

    /**
     * Muestra el nombre del trabajador actual
     * @return El nombre del trabajador
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public String muestra_nombre_trabajador() throws ClassNotFoundException, SQLException {
        Cursor resultado = db_contenedores.rawQuery("SELECT nombre FROM trabajador WHERE id_trabajador='" + id_trabajador_sesion + "'", null);
        resultado.moveToNext();
        return resultado.getString(0);
    }

    /**
     * Muestra los consignatarios
     * @return Los valores de los consignatarios
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public String muestra_consignatario() throws ClassNotFoundException, SQLException {
        StringBuilder option_consignatario = new StringBuilder();
        Cursor resultado = db_contenedores.rawQuery("SELECT id_consignatario, nombre FROM consignatario", null);
        while (resultado.moveToNext()) {
            option_consignatario.append("<option value=\"")
                    .append(resultado.getString(0))
                    .append("\">")
                    .append(resultado.getString(1))
                    .append("</option>").append("\n");
        }
        return option_consignatario.toString();
    }

    /**
     * Calcula el dígito de control de la matrícula del contenedor
     * @param matricula_texto  Los 3 primeros caracteres de la matrícula
     * @param matricula_u      El carácter U de la matrícula
     * @param matricula_numero Los 6 primeros dígitos de la matrícula
     * @see "http://mundopuerto.blogspot.com.es/2009/09/calculo-del-digito-de-control-en-la.html"
     * @see "https://www.bic-code.org/calculate-the-check-digit-online.html"
     * @return El dígito de control
     */
    @JavascriptInterface
    public int matricula_calcula_digito_control(String matricula_texto, String matricula_u, String matricula_numero) {
        matricula_texto = matricula_texto.toUpperCase();
        Map<Character, Integer> valores = new HashMap<>(); // Creamos HashMap con los valores recogidos de http://mundopuerto.blogspot.com/2009/09/calculo-del-digito-de-control-en-la.html
        valores.put('A', 10); valores.put('B', 12); valores.put('C', 13); valores.put('D', 14); valores.put('E', 15); valores.put('F', 16); valores.put('G', 17);
        valores.put('H', 18); valores.put('I', 19); valores.put('J', 20); valores.put('K', 21); valores.put('L', 23); valores.put('M', 24); valores.put('N', 25);
        valores.put('O', 26); valores.put('P', 27); valores.put('Q', 28); valores.put('R', 29); valores.put('S', 30); valores.put('T', 31); valores.put('U', 32);
        valores.put('V', 34); valores.put('W', 35); valores.put('X', 36); valores.put('Y', 37); valores.put('Z', 38);
        int valor0 = (int) (valores.get(matricula_texto.charAt(0)) * Math.pow(2,0));
        int valor1 = (int) (valores.get(matricula_texto.charAt(1)) * Math.pow(2,1));
        int valor2 = (int) (valores.get(matricula_texto.charAt(2)) * Math.pow(2,2));
        int valor3 = (int) (valores.get(matricula_u.charAt(0)) * Math.pow(2,3));
        int valor4 = (int) (Integer.parseInt(String.valueOf(matricula_numero.charAt(0))) * Math.pow(2, 4));
        int valor5 = (int) (Integer.parseInt(String.valueOf(matricula_numero.charAt(1))) * Math.pow(2, 5));
        int valor6 = (int) (Integer.parseInt(String.valueOf(matricula_numero.charAt(2))) * Math.pow(2, 6));
        int valor7 = (int) (Integer.parseInt(String.valueOf(matricula_numero.charAt(3))) * Math.pow(2, 7));
        int valor8 = (int) (Integer.parseInt(String.valueOf(matricula_numero.charAt(4))) * Math.pow(2, 8));
        int valor9 = (int) (Integer.parseInt(String.valueOf(matricula_numero.charAt(5))) * Math.pow(2, 9));
        int suma = valor0 + valor1 + valor2 + valor3 + valor4 + valor5 + valor6 + valor7 + valor8 + valor9;
        int modulo = suma % 11;
        char control = (Integer.toString(modulo).charAt(Integer.toString(modulo).length()-1));
        control = (char) Integer.parseInt(String.valueOf(control));
        return control;
    }

    /**
     * Inserta una operación
     * @param matricula_texto   Los 3 primeros caracteres de la matrícula
     * @param matricula_u       El carácter U de la matrícula
     * @param matricula_numero  Los 6 primeros dígitos de la matrícula
     * @param matricula_control El dígito de control de la matrícula
     * @param origen            El origen del contenedor
     * @param destino           El destino del contenedor
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public void inserta_operacion(String id_consignatario, String matricula_texto, String matricula_u, String matricula_numero, String matricula_control, String vacio, String origen, String destino) throws ClassNotFoundException, SQLException {
        String matricula = matricula_texto.toUpperCase() + matricula_u + matricula_numero + matricula_control;
        Date date = new Date();
        String fecha = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date);
        db_contenedores.execSQL("INSERT INTO operacion (id_trabajador, id_consignatario, matricula, vacio, origen, destino, fecha) VALUES ('" + id_trabajador_sesion + "', '" + id_consignatario + "', '" + matricula + "', '" + vacio + "', '" + origen + "', '" + destino + "', '" + fecha + "')");
    }

    /**
     * Muestra el número de las últimas operaciones
     * @return El número de las últimas operaciones
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public int muestra_numero_ultimas_operaciones() throws ClassNotFoundException, SQLException {
        int numero = 0;
        Cursor resultado = db_contenedores.rawQuery("SELECT * FROM operacion WHERE id_trabajador='" + id_trabajador_sesion + "'", null);
        if (resultado.moveToNext()) {
            numero = resultado.getCount();
        }
        return numero;
    }

    /**
     * Muestra las 5 últimas operaciones
     * @return Tabla que muestra las 5 últimas operaciones introducidas
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public String muestra_ultimas_operaciones() throws ClassNotFoundException, SQLException {
        String id_operacion;
        int tabla_pijama = 0;
        StringBuilder ultimas_operaciones = new StringBuilder();
        Cursor resultado = db_contenedores.rawQuery("SELECT * FROM operacion WHERE id_trabajador='" + id_trabajador_sesion + "' ORDER BY fecha DESC LIMIT 5", null);
        while (resultado.moveToNext()) {
            String id_consignatario = resultado.getString(2);
            String consignatario;
            Cursor resultado_consignatario = db_contenedores.rawQuery("SELECT nombre FROM consignatario WHERE id_consignatario='" + id_consignatario + "'", null);
            resultado_consignatario.moveToNext();
            consignatario = resultado_consignatario.getString(0);
            if (tabla_pijama==0) {
                id_operacion = resultado.getString(0);
                ultimas_operaciones.append("<tr>\n")
                        .append("<td align=\"center\">")
                        .append(consignatario)
                        .append("</td>\n")
                        .append("<td align=\"center\">")
                        .append(resultado.getString(3))
                        .append("</td>\n")
                        .append("<td align=\"center\">")
                        .append(resultado.getString(4))
                        .append("</td>\n")
                        .append("<td align=\"center\">")
                        .append(resultado.getString(5))
                        .append("</td>\n")
                        .append("<td align=\"center\">")
                        .append(resultado.getString(6))
                        .append("</td>\n")
                        .append("<td align=\"center\">")
                        .append(resultado.getString(7).split("\\.")[0])
                        .append("</td>\n")
                        .append("<td align=\"center\"> <input type=\"image\" name=\"eliminar\" value=\"Eliminar\" src=\"./template/borrar.png\" width=\"20\" onclick=\"javascript:elimina_operacion(")
                        .append(id_operacion)
                        .append(");\"> </td>\n")
                        .append("</tr>\n");
                tabla_pijama = 1;
            } else {
                id_operacion = resultado.getString(0);
                ultimas_operaciones.append("<tr>\n")
                        .append("<td align=\"center\" bgcolor=\"#E6E6E6\">")
                        .append(consignatario)
                        .append("</td>\n")
                        .append("<td align=\"center\" bgcolor=\"#E6E6E6\">")
                        .append(resultado.getString(3))
                        .append("</td>\n")
                        .append("<td align=\"center\" bgcolor=\"#E6E6E6\">")
                        .append(resultado.getString(4))
                        .append("</td>\n")
                        .append("<td align=\"center\" bgcolor=\"#E6E6E6\">")
                        .append(resultado.getString(5))
                        .append("</td>\n")
                        .append("<td align=\"center\" bgcolor=\"#E6E6E6\">")
                        .append(resultado.getString(6))
                        .append("</td>\n")
                        .append("<td align=\"center\" bgcolor=\"#E6E6E6\">")
                        .append(resultado.getString(7).split("\\.")[0])
                        .append("</td>\n")
                        .append("<td align=\"center\" bgcolor=\"#E6E6E6\"> <input type=\"image\" name=\"eliminar\" value=\"Eliminar\" src=\"./template/borrar.png\" width=\"20\" onclick=\"javascript:elimina_operacion(")
                        .append(id_operacion)
                        .append(");\"> </td>\n")
                        .append("</tr>\n");
                tabla_pijama = 0;
            }
        }
        return ultimas_operaciones.toString();
    }

    /**
     * Elimina una operación
     * @param id_operacion El id de la operación a eliminar
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public void elimina_operacion(final String id_operacion) {
        Cursor resultado = db_contenedores.rawQuery("SELECT * FROM operacion WHERE id_operacion = " + id_operacion, null);
        resultado.moveToNext();
        String id_consignatario = resultado.getString(2);
        String consignatario;
        Cursor resultado_consignatario = db_contenedores.rawQuery("SELECT nombre FROM consignatario WHERE id_consignatario='" + id_consignatario + "'", null);
        resultado_consignatario.moveToNext();
        consignatario = resultado_consignatario.getString(0);
        AlertDialog.Builder alerta = new AlertDialog.Builder(contexto);
        alerta.setTitle("AVISO - Eliminar operación");
        alerta.setMessage("Vas a eliminar la siguiente operación:\n\n - Consignatario: " + consignatario + "\n - Matrícula: " + resultado.getString(3) + "\n - Contenedor vacío: " + resultado.getString(4)  + "\n - Origen: " + resultado.getString(5) + "\n - Destino: " + resultado.getString(6) + "\n - Fecha: " + resultado.getString(7).split("\\.")[0] + "\n\n¿Estás seguro?");
        alerta.setPositiveButton("SI", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                db_contenedores.execSQL("DELETE FROM operacion WHERE id_operacion = " + id_operacion);
                contenedores.post(new Runnable() {
                    @Override
                    public void run() {
                        contenedores.reload(); // Recargamos la vista tras la eliminación de un registro
                    }
                });
            }
        });
        alerta.setNegativeButton("NO", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                muestra_mensaje("ELIMINACIÓN CANCELADA");
            }
        });
        alerta.show();
    }

    /**
     * Muestra diálogo de confirmación para finalizar sesión
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public void finaliza_sesion() throws ClassNotFoundException, SQLException {
        AlertDialog.Builder alerta = new AlertDialog.Builder(contexto);
        alerta.setTitle("AVISO - Finalizar sesión");
        alerta.setMessage("Vas a finalizar la sesión de trabajo\n\nLas operaciones se guardarán de manera permanente\nen la base de datos\n\n¿Estás seguro?");
        alerta.setPositiveButton("SI", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                Cursor resultado = db_contenedores.rawQuery("SELECT * FROM operacion WHERE id_trabajador = '" + id_trabajador_sesion + "'", null);
                if (resultado.moveToNext()) {
                    contenedores.post(new Runnable() { // Si finalizamos sesión y hay operaciones que enviar al servidor remoto, cargamos el html que inicia el proceso
                        @Override
                        public void run() {
                            contenedores.loadUrl("file:///android_asset/loading.html");
                        }
                    });
                } else {
                    contenedores.post(new Runnable() { //Si finalizamos sesión y no hay operaciones que enviar al servidor remoto, volvemos a la página inicial
                        @Override
                        public void run() {
                            contenedores.loadUrl("file:///android_asset/index.html");
                        }
                    });
                }
            }
        });
        alerta.setNegativeButton("NO", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                muestra_mensaje("FINALIZACIÓN DE SESIÓN CANCELADA");
            }
        });
        alerta.show();
    }

    /**
     * Conecta a la base de datos de APML y realiza un volcado de la base de datos local
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    @JavascriptInterface
    public void volcar_operaciones() throws ClassNotFoundException, SQLException {
        Cursor resultado = db_contenedores.rawQuery("SELECT * FROM operacion WHERE id_trabajador = '" + id_trabajador_sesion + "'", null);
        try {
            Class.forName("com.mysql.jdbc.Driver");
            Connection conexion = DriverManager.getConnection("jdbc:mysql://192.168.100.144/contenedorespruebas", "contenedor", "contenedor");
            Statement insercion = conexion.createStatement(); // Establecemos conexión con los datos proporcionados
            while (resultado.moveToNext()) {
                insercion.executeUpdate("INSERT INTO operacion (id_trabajador, id_consignatario, matricula, vacio, origen, destino, fecha) VALUES ('" + resultado.getString(1) + "', '" + resultado.getString(2) + "', '" + resultado.getString(3) + "', '" + resultado.getString(4) + "', '" + resultado.getString(5) + "', '" + resultado.getString(6) + "', '" + resultado.getString(7) + "')");
                db_contenedores.execSQL("DELETE FROM operacion WHERE id_operacion = '" + resultado.getString(0) + "'"); // Una vez enviadas correctamente las operaciones, se vacía la tabla que las almacena
            }
            conexion.close();
        } catch (ClassNotFoundException | SQLException e) {
            muestra_alerta("AVISO - No hay conexión", "Posiblemente la tablet no tenga conexión a Internet\n\nLas operaciones permanecerán cargadas en la tablet hasta que la conexión se restablezca y se finalice sesión nuevamente", "ACEPTAR");
            e.printStackTrace();
        }
        contenedores.post(new Runnable() { // Tras finalizar el proceso de volcado de datos, cargamos en la vista la página inicial
            @Override
            public void run() {
                contenedores.loadUrl("file:///android_asset/index.html");
            }
        });
    }
}
